#pragma once

#include "stdafx.h"

using namespace std;

class ScripterStream
{
public:

	const uint8_t RECT_BRACKET = 0x01;
	const uint8_t TRI_BRACKET = 0x02;
  	const uint8_t READ_MODE = 0x00;
	const uint8_t WRITE_MODE = 0x01;

	//Constructor
	ScripterStream();
	~ScripterStream();
	
	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    startScripter
	*Description: open the script file and get the path directory
	*Parameters:  none
	*Returns:     std::string retPathDir: Path directory of executable file
	*******************************************************************************/
	std::string startScripter(char * ch);
	
	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    getLine
	*Description: check if command still available or end of file is reached
	*Parameters:  none
	*Returns:     bool true: line command available
	*             bool false: end of file reach
	*******************************************************************************/
	bool getLine();

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    retStringCommand
	*Description: return string line from the script file
	*Parameters:  none
	*Returns:     std::string line: command
	*******************************************************************************/
	std::string retStringCommand();

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    retSendStatus
	*Description: initiate the pointer to streaming input file
	*Parameters:  ActualCommandParams actualCommandParam: get the input file
	*             std::string mainPathDir: path directory of exe and input file
	*Returns:     none
	*             exception if the file does not exist
	*******************************************************************************/
	void initReadTxtFile(ActualCommandParams actualCommandParams, std::string mainPathDir);

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    readTxtFile
	*Description: close the streaming input file
	*Parameters:  ActualCommandParams actualCommandParams: get the
	*Returns:     none
	*******************************************************************************/
	void readTxtFile();

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    retDataBlockRead
	*Description: read and copy data from streaming input
	*Parameters:  none
	*Returns:     DataBlock dataBlockRead: data to be transmitted to BSL device
	*******************************************************************************/
	DataBlock retDataBlockRead();

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    retReadStatus
	*Description: return read status
	*Parameters:  none
	*Returns:     uint8_t readStatus: READ_ON_GOING
	*                                 READ_DONE
	*******************************************************************************/
	uint8_t retReadStatus();
	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    retSendStatus
	*Description: assign clear send status
	*Parameters:  none
	*Returns:     uint8_t retSendStatus: SEND_DATA_BLOCK
	*                                    KEEP_DATA_BLOCK
	*******************************************************************************/
	uint8_t retSendStatus();

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    closeReadTxtFile
	*Description: close the streaming input file
	*Parameters:  none
	*Returns:     none
	*******************************************************************************/
	void closeReadTxtFile();

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    initWriteTxtFile
	*Description: create new file to write data
	*Parameters:  ActualCommandParams actualCommandParams: parameter from the interpreter
	*             std::string mainPathDir : location of script file located
	*Returns:     none
	*******************************************************************************/
	void initWriteTxtFile(ActualCommandParams actualCommandParams, std::string mainPathDir);
	
	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    writeTxtFile
	*Description: write data to the txt file
	*Parameters:  uint8_t *buf  : pointer to dataBuffer which data will be written
	*                             to the file
	*			  uint32_t size	: number of bytes will be written to the file
	*Returns:	  none
	*******************************************************************************/
	void writeTxtFile(uint8_t *buf, uint32_t size);

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:	  resetBytesWritten
	*Description: reset the count bytes of how many bytes already written to the file
	*Parameters:  none
	*Returns:	  none
	*******************************************************************************/
	void resetBytesWritten();

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:	  retBytesWritten
	*Description: return the count bytes of how many bytes already written to the file
	*Parameters:  none
	*Returns:	  uint32_t : number bytes written to the file
	*******************************************************************************/
	uint32_t retBytesWritten();

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    closeWriteTxtFile
	*Description: close the streaming output file
	*Parameters:  none
	*Returns:     none
	*******************************************************************************/
	void closeWriteTxtFile();
  	
	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    initLogTxtFile
	*Description: create log file
	*Parameters:  bool enLogInterpreter: will not generate log file if bool false
	*             std::string path : location of script file located
	*Returns:     none
	*******************************************************************************/
	void initLogTxtFile(bool enLogInterpreter, std::string path);

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    writeLogFile
	*Description: write data to the log
	*Parameters:  std::string line : string to be written
	*			  bool tab	: tab('\t') applied
	*Returns:	  none
	*******************************************************************************/
	void writeLogFile(std::string line, bool tab);

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    writeVerbose
	*Description: write the bytes sent and received received
	*Parameters:  uint8_t *buf  : pointer to dataBuffer which data will be written
	*                             to the file
	*			  uint32_t bufSize	: number of bytes will be written to the file
	*             uint8_t bracket : bracket type use for the verbose
	*Returns:	  none
	*******************************************************************************/
	void writeVerbose(uint8_t *buf, uint32_t bufSize, uint8_t bracket);

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    closeLogFile
	*Description: close the streaming log file
	*Parameters:  none
	*Returns:     none
	*******************************************************************************/
	void closeLogFile();

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    calcTransSpeed
	*Description: calculate the speed of the transmission
	*Parameters:  double time : total time elapsed
	*			  uint32_t numOfbytes : number of pure data bytes on the transmission
	*			  uint8_t mode : choose if it's reading/writing mode
	*Returns:     none
	*******************************************************************************/
	void calcTransSpeed(double time, uint32_t numOfbytes, uint8_t mode);
	
private:

	const uint8_t SEND_DATA_BLOCK = 1;
	const uint8_t KEEP_DATA_BLOCK = 0;
	const uint8_t READ_ON_GOING = 1;
	const uint8_t READ_DONE = 0;
	const uint8_t CHANGE_ADDRESS = 1;
	const uint8_t NO_CHANGE_ADDRESS = 0;

	//Streaming scripter
	std::string fileScriptName;
	std::ifstream fileScriptStream;
	std::string line;
	boost::posix_time::ptime date_time;
	std::string showHeaderFile;

	//File pointer for reading and writing
	std::string fileName;

	//Reading task
	std::ifstream fileReadStream;
	uint16_t bytesRead;
	uint32_t saveLastReadAddress;
	uint16_t numBytesToRead;
	DataBlock dataBlockRead;
	uint32_t  maxBufSize;

	uint8_t readStatus;
	uint8_t changeAddress;
	uint8_t sendStatus;

	//Writing task
	std::ofstream fileWriteStream;
	uint32_t bytesWritten;
	uint32_t k;
	
	//Logging task
	bool enLog;
	std::ofstream fileLogStream;

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    getThePathFile
	*Description: set the path for the respective file
	*Parameters:  none
	*Returns:     std::string resultPath: complete streaming file
	*******************************************************************************/
 	std::string getThePathFile(std::string stInput, std::string relativePath);
	
	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    setSendStatus
	*Description: assign clear send status
	*Parameters:  none
	*Returns:     none
	*******************************************************************************/
	void clearSendStatus();
	
	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    setSendStatus
	*Description: assign set send status
	*Parameters:  none
	*Returns:     none
	*******************************************************************************/
	void setSendStatus();
	
	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    onGoingReadStatus
	*Description: assign read status from streaming file
	*Parameters:  none
	*Returns:     none
	*******************************************************************************/
	void onGoingReadStatus();
	
	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    finishReadStatus
	*Description: assign finish read status from streaming file
	*Parameters:  none
	*Returns:     none
	*******************************************************************************/
	void finishReadStatus();
  
	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    intToHex
	*Description: convert the uint8_t type into hex in string type
	*             need to cast the uint8_t to uint32_t
	*Parameters:  uint8_t input: integer input to be converted to hex
	*Returns:     std::string  stream.str(): the string format of hex value
	*******************************************************************************/
	std::string intToHex(uint8_t input);

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    intToHexAddr
	*Description: convert the uint32_t type into hex in string type
	*Parameters:  uint8_t input: integer input to be converted to hex
	*Returns:     std::string  stream.str(): the string format of hex value
	*******************************************************************************/
	std::string intToHexAddr(uint32_t address);
};


